from .absence import (
    AbsenceRequestCrossBorderCountryModelSerializer,
    AbsenceRequestModelSerializer,
    AbsenceRequestPeriodsModelSerializer,
    AbsenceRequestTypeModelSerializer,
    AbsenceRequestTypeRepresentationSerializer,
    EmployeeAbsenceDaysModelSerializer,
    IncreaseDaySerializer,
)
from .calendars import (
    DayOffCalendarModelSerializer,
    DayOffCalendarRepresentationSerializer,
    DayOffModelSerializer,
    DayOffRepresentationSerializer,
    DefaultDailyPeriodModelSerializer,
    DefaultDailyPeriodRepresentationSerializer,
    EmployeeWeeklyOffPeriodsRepresentationSerializer,
)
from .employee import (
    DeactivateEmployeeSerializer,
    EmployeeBalanceModelSerializer,
    EmployeeHumanResourceRepresentationSerializer,
    EmployeeModelSerializer,
    EmployeeWeeklyOffPeriodsModelSerializer,
    EmployeeYearBalanceModelSerializer,
    EmployeeYearBalanceRepresentationSerializer,
    PositionModelSerializer,
    PositionRepresentationSerializer,
)
from .kpi import (
    EvaluationModelSerializer,
    EvaluationRepresentationSerializer,
    KPIModelSerializer,
    KPIRepresentationSerializer,
)
from .review import (
    ReviewAnswerModelSerializer,
    ReviewAnswerRepresentationSerializer,
    ReviewGroupModelSerializer,
    ReviewGroupRepresentationSerializer,
    ReviewListModelSerializer,
    ReviewModelSerializer,
    ReviewQuestionCategoryModelSerializer,
    ReviewQuestionCategoryRepresentationSerializer,
    ReviewQuestionModelSerializer,
    ReviewQuestionRepresentationSerializer,
    ReviewReadOnlyModelSerializer,
    ReviewRepresentationSerializer,
)



from django.utils.translation import gettext_lazy
from wbcore.contrib.icons import WBIcon
from wbcore.enums import RequestType
from wbcore.metadata.configs import buttons as bt
from wbcore.metadata.configs.buttons.view_config import ButtonViewConfig
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    create_simple_display,
)

from wbhuman_resources.serializers import DeactivateEmployeeSerializer


class EmployeeButtonConfig(ButtonViewConfig):
    def get_custom_instance_buttons(self):
        return {
            bt.ActionButton(
                method=RequestType.PATCH,
                identifiers=("wbhuman_resources:employee",),
                key="deactivate",
                action_label=gettext_lazy("Deactivating Employee"),
                label=gettext_lazy("Deactivate Employee"),
                icon=WBIcon.DISABLED.icon,
                title=gettext_lazy("Deactivate Employee"),
                serializer=DeactivateEmployeeSerializer,
                description_fields=gettext_lazy(
                    """
                    <p>You are about to deactivate the employee <b>{{computed_str}}</b>. This will disable this employee's and user's accounts. If you want to transfer relationships to a substitute, please select it in the list bellow:</p>
                """
                ),
                instance_display=create_simple_display([["substitute"]]),
            ),
            bt.WidgetButton(key="balance_and_usage", label="Balance & Usage", icon=WBIcon.DATA_GRID.icon),
        }

    def get_custom_list_instance_buttons(self):
        return self.get_custom_instance_buttons()


class YearBalanceEmployeeHumanResourceButtonConfig(ButtonViewConfig):
    def get_custom_list_instance_buttons(self):
        return super().get_custom_list_instance_buttons()

    def get_custom_instance_buttons(self):
        return super().get_custom_instance_buttons()



import zoneinfo
from datetime import time

import factory
from faker import Faker

from wbhuman_resources.models.calendars import (
    DayOff,
    DayOffCalendar,
    DefaultDailyPeriod,
)

fake = Faker()


class BaseDayOffCalendarFactory(factory.django.DjangoModelFactory):
    title = "Default Calendar"
    resource = "europe.Germany"
    timezone = zoneinfo.ZoneInfo("UTC")

    class Meta:
        model = DayOffCalendar
        django_get_or_create = ["resource"]


class DayOffCalendarFactory(BaseDayOffCalendarFactory):
    @factory.post_generation
    def post_gen(self, create, extracted, **kwargs):
        if create:
            DefaultDailyPeriodFactory.create(calendar=self)
            DefaultDailyPeriodFactory.create(
                calendar=self,
                lower_time=time(
                    14,
                    0,
                    0,
                ),
                upper_time=time(18, 0, 0),
                title="afternoon",
            )


class DefaultDailyPeriodFactory(factory.django.DjangoModelFactory):
    lower_time = time(9, 0, 0)
    upper_time = time(13, 0, 0)
    title = "morning"
    calendar = factory.SubFactory("wbhuman_resources.factories.BaseDayOffCalendarFactory")

    class Meta:
        model = DefaultDailyPeriod
        django_get_or_create = ["lower_time", "upper_time", "calendar"]


class DayOffFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = DayOff

    title = factory.Faker("text", max_nb_chars=64)
    date = factory.Faker("date_object")
    calendar = factory.SubFactory("wbhuman_resources.factories.DayOffCalendarFactory")



from io import StringIO

import pandas as pd
from django.contrib import admin
from django.shortcuts import redirect, render
from django.utils.translation import gettext_lazy as _
from reversion.errors import RevertError
from wbcore.admin import CsvImportForm, ImportCsvMixin

from ..models.absence import AbsenceRequest, AbsenceRequestPeriods, AbsenceRequestType


class CustomImportCsvMixin(ImportCsvMixin):
    def _import_csv(self, request, _sep=";"):
        if request.method == "POST":
            csv_file = request.FILES["csv_file"]

            str_text = ""
            for line in csv_file:
                str_text = str_text + line.decode()
            # Import csv as df
            df = pd.read_csv(StringIO(str_text), sep=_sep)
            # Sanitize dataframe
            df = df.where(pd.notnull(df), None)
            df = df.drop(df.columns.difference(self.get_import_fields()), axis=1)

            # Overide this function if there is foreign key ids in the dataframe
            df = self.manipulate_df(df)
            errors = 0
            revert_errors = 0
            nb_added = 0
            for model in df.to_dict("records"):
                # by default, process the modela as a create request. Can be override to change the behavior
                try:
                    nb_added += self.process_model(model)
                # https://django-reversion.readthedocs.io/en/stable/common-problems.html
                except RevertError:
                    revert_errors += 1
                except Exception as e:
                    print(e)  # noqa: T201
                    errors += 1
            self.message_user(
                request,
                _(
                    "Your CSV file has been imported : {} imported ({} added, {} updated), {} errors, {} revert errors found due to failure to restore old versions"
                ).format(
                    df.shape[0] - errors - revert_errors,
                    nb_added,
                    df.shape[0] - errors - revert_errors - nb_added,
                    errors,
                    revert_errors,
                ),
            )
            return redirect("..")
        form = CsvImportForm()
        payload = {"form": form}
        return render(request, "wbcore/admin/csv_form.html", payload)


@admin.register(AbsenceRequestType)
class AbsenceRequestTypeAdmin(admin.ModelAdmin):
    list_display = [
        "title",
        "is_vacation",
        "is_timeoff",
        "is_extensible",
        "auto_approve",
        "days_in_advance",
    ]

    autocomplete_fields = ["crossborder_countries", "extra_notify_groups"]


class AbsenceRequestPeriodsInline(admin.TabularInline):
    model = AbsenceRequestPeriods
    ordering = ("timespan__startswith",)
    extra = 0
    raw_id_fields = ["balance", "request"]


@admin.register(AbsenceRequest)
class AbsenceRequestAdmin(admin.ModelAdmin):
    list_display = ["status", "employee", "period", "type", "_total_hours", "_total_vacation_hours"]

    fieldsets = (
        (
            "",
            {
                "fields": (
                    ("status", "type"),
                    ("period", "employee"),
                    "attachment",
                )
            },
        ),
        (
            _("Notes"),
            {"fields": ("notes", "reason")},
        ),
    )
    inlines = (AbsenceRequestPeriodsInline,)
    autocomplete_fields = ["employee"]
    list_filter = ["type__is_vacation"]

    def _total_hours(self, obj):
        return obj._total_hours

    def _total_vacation_hours(self, obj):
        return obj._total_vacation_hours

    #
    # def get_queryset(self, request):
    #     return super().get_queryset().



from dataclasses import dataclass
from datetime import date

from django.contrib.messages import info
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from rest_framework.reverse import reverse
from wbcore import serializers as wb_serializers
from wbcore.contrib.authentication.serializers import GroupRepresentationSerializer
from wbcore.contrib.geography.serializers import CountryRepresentationSerializer

from wbhuman_resources.models import (
    AbsenceRequest,
    AbsenceRequestPeriods,
    AbsenceRequestType,
    EmployeeHumanResource,
)

from .calendars import DefaultDailyPeriodRepresentationSerializer
from .employee import (
    EmployeeHumanResourceRepresentationSerializer,
    EmployeeYearBalanceRepresentationSerializer,
    PositionRepresentationSerializer,
)


class AbsenceRequestTypeModelSerializer(wb_serializers.ModelSerializer):
    _extra_notify_groups = GroupRepresentationSerializer(source="extra_notify_groups", many=True)

    @wb_serializers.register_only_instance_resource()
    def crossborder_countries_additional_resources(self, instance, request, user, **kwargs):
        return {
            "crossbordercountries": reverse(
                "wbhuman_resources:absencerequesttype-crossbordercountry-list",
                args=[instance.id],
                request=request,
            )
        }

    class Meta:
        model = AbsenceRequestType
        fields = [
            "id",
            "title",
            "is_vacation",
            "is_timeoff",
            "is_extensible",
            "auto_approve",
            "days_in_advance",
            "is_country_necessary",
            "extra_notify_groups",
            "_extra_notify_groups",
            "icon",
            "color",
            "_additional_resources",
        ]


class AbsenceRequestTypeRepresentationSerializer(wb_serializers.RepresentationSerializer):
    endpoint = "wbhuman_resources:absencerequesttyperepresentation-list"
    _detail = wb_serializers.HyperlinkField(reverse_name="wbhuman_resources:absencerequesttype-detail")

    class Meta:
        model = AbsenceRequestType
        fields = [
            "id",
            "title",
            "_detail",
        ]


class AbsenceRequestCrossBorderCountryModelSerializer(wb_serializers.ModelSerializer):
    geography_repr = wb_serializers.CharField(read_only=True)
    _geography = CountryRepresentationSerializer(source="geography")

    class Meta:
        model = AbsenceRequestType.crossborder_countries.through
        fields = ("id", "geography_repr", "_geography", "geography", "absencerequesttype")


@dataclass
class CurrentUserDefaultPeriodDateTimeRange:
    requires_context = True
    user_attr = "profile.human_resources"

    def __call__(self, serializer_instance):
        employee = wb_serializers.CurrentUserDefault("profile.human_resources")(serializer_instance)
        if employee:
            return employee.calendar.get_default_fullday_period(date.today())


def get_lower_time_choices(field, request):
    if (profile := request.user.profile) and (employee := getattr(profile, "human_resources", None)):
        return employee.calendar.get_period_start_choices()
    return []


def get_upper_time_choices(field, request):
    if (profile := request.user.profile) and (employee := getattr(profile, "human_resources", None)):
        return employee.calendar.get_period_end_choices()
    return []


class AbsenceRequestModelSerializer(wb_serializers.ModelSerializer):
    _type = AbsenceRequestTypeRepresentationSerializer(source="type")
    employee = wb_serializers.PrimaryKeyRelatedField(
        default=wb_serializers.CurrentUserDefault(user_attr="profile.human_resources"),
        queryset=EmployeeHumanResource.active_internal_employees.all(),
        read_only=lambda view: not view.request.user.has_perm("wbhuman_resources.administrate_absencerequest"),
    )
    _employee = EmployeeHumanResourceRepresentationSerializer(source="employee")
    period = wb_serializers.DateTimeRangeField(
        default=CurrentUserDefaultPeriodDateTimeRange(),
        lower_time_choices=get_lower_time_choices,
        upper_time_choices=get_upper_time_choices,
    )

    created = wb_serializers.DateTimeField(read_only=True)
    reason = wb_serializers.TextField(label="Reason for refusal", read_only=lambda view: not view.can_administrate)

    _total_hours = wb_serializers.FloatField(
        label=_("Total Hours"),
        required=False,
        read_only=True,
        help_text=_("The total number of hours this request spans (i.e. only actual working days)"),
    )
    _total_vacation_hours = wb_serializers.FloatField(
        label=_("Vacation Hours"),
        required=False,
        read_only=True,
        help_text=_("The number of vacation hours this request uses"),
    )

    _total_hours_in_days = wb_serializers.FloatField(
        label=_("Total Days"),
        required=False,
        read_only=True,
        help_text=_("The total number of days this request spans (i.e. only actual working days)"),
    )
    _total_vacation_hours_in_days = wb_serializers.FloatField(
        label=_("Vacation Days"),
        required=False,
        read_only=True,
        help_text=_("The number of vacation days this request uses"),
    )

    department = wb_serializers.PrimaryKeyRelatedField(read_only=True)
    _department = PositionRepresentationSerializer(source="department")

    _crossborder_country = CountryRepresentationSerializer(source="crossborder_country")

    @wb_serializers.register_only_instance_resource()
    def additional_resources(self, instance, request, user, **kwargs):
        res = {}
        res["periods"] = reverse(
            "wbhuman_resources:request-periods-list",
            args=[instance.id],
            request=request,
        )
        if instance and instance.type.is_extensible and instance.status == AbsenceRequest.Status.APPROVED.name:
            if not instance.next_extensible_period:
                info(
                    request,
                    _(
                        "This request cannot be extended because the next available date is already taken. Please amend this request first"
                    ),
                )
            else:
                res["increase_days"] = reverse(
                    "wbhuman_resources:absencerequest-increaseday",
                    args=[instance.id],
                    request=request,
                )
        return res

    def validate(self, data):
        errors = {}
        obj = self.instance
        request_type = data.get("type", obj.type if obj else None)
        if not request_type:
            raise serializers.ValidationError({"type": [_("A type needs to be provided")]})
        # check crossborder rules
        try:
            request_type.validate_country(data.get("crossborder_country", obj.crossborder_country if obj else None))
        except ValueError as e:
            errors["crossborder_country"] = e.args[0]

        # If the user does not have the permission to administrate all the vacation requests,
        # then we need to set the profile here, because this is, in this case, a read only field.
        # if obj already exist the owner employee is not updated, important for keep the same employee when the manager update the object
        if (
            data
            and (request := self.context.get("request"))
            and (user_employee := getattr(request.user.profile, "human_resources", None))
        ):
            if not request.user.has_perm("wbhuman_resources.administrate_absencerequest") and not obj:
                data["employee"] = user_employee

            employee = data.get("employee", obj.employee if obj else None)
            period = data.get("period", obj.period if obj else None)

            # Check if the period is a valid datetime range
            if period.lower >= period.upper:
                errors["period"] = [gettext("End date cannot be before start date")]

            # If requester is not a manager of the request's employee, we check if the request can be taken given the time rules
            if not user_employee.is_manager_of(employee) and not EmployeeHumanResource.is_administrator(request.user):
                now = timezone.now()
                if period.lower < now or period.upper < now:
                    errors["period"] = [
                        gettext("You cannot save or modify an absence already started or already finished")
                    ]
                if (period.lower.date() - now.date()).days < request_type.days_in_advance:
                    errors["period"] = [
                        gettext("The request needs to start at least {days_in_advance} days from now").format(
                            days_in_advance=request_type.days_in_advance
                        )
                    ]

            # Check if overlapping requests exists
            if (
                AbsenceRequest.objects.exclude(
                    status__in=[AbsenceRequest.Status.CANCELLED, AbsenceRequest.Status.DENIED]
                )
                .filter(
                    models.Q(employee=employee)
                    & ~models.Q(id=obj.id if obj else -1)  # TODO: Remove later - Not really needed (probably)
                    & models.Q(period__overlap=period)
                )
                .exists()
            ):
                errors["non_field_errors"] = [
                    _("Overlaping requests already exists at the specified periods. Please change the dates.")
                ]
            if len(errors.keys()) > 0:
                raise serializers.ValidationError(errors)

        return data

    class Meta:
        model = AbsenceRequest
        fields = [
            "id",
            "status",
            "employee",
            "_employee",
            "period",
            "type",
            "_type",
            "notes",
            "reason",
            "_total_hours",
            "_total_vacation_hours",
            "_total_hours_in_days",
            "_total_vacation_hours_in_days",
            "created",
            "attachment",
            "department",
            "_department",
            "_additional_resources",
            "is_cancelled",
            "_crossborder_country",
            "crossborder_country",
        ]


class EmployeeAbsenceDaysModelSerializer(wb_serializers.ModelSerializer):
    year = wb_serializers.YearField(read_only=True)
    absence_type = wb_serializers.ChoiceField(read_only=True, choices=AbsenceRequestType.get_choices())
    hours_count = wb_serializers.FloatField(read_only=True)
    days_count = wb_serializers.FloatField(read_only=True)
    id = wb_serializers.PrimaryKeyCharField()

    class Meta:
        model = AbsenceRequestPeriods
        fields = ["id", "absence_type", "year", "hours_count", "days_count"]


class AbsenceRequestPeriodsModelSerializer(wb_serializers.ModelSerializer):
    _balance = EmployeeYearBalanceRepresentationSerializer(source="balance")
    _default_period = DefaultDailyPeriodRepresentationSerializer(source="default_period")
    _total_hours = wb_serializers.FloatField(read_only=True, required=False)

    class Meta:
        model = AbsenceRequestPeriods
        fields = [
            "id",
            "request",
            "employee",
            "default_period",
            "_default_period",
            "date",
            "timespan",
            "_total_hours",
            "balance",
            "_balance",
            "consecutive_hours_count",
        ]
        read_only_fields = fields


class IncreaseDaySerializer(wb_serializers.Serializer):
    number_days = wb_serializers.IntegerField(label=_("Increase absence by"), default=1)



from datetime import date
from typing import Optional

from django.utils.translation import gettext as _
from wbcore.contrib.color.enums import WBColor
from wbcore.enums import Unit
from wbcore.metadata.configs import display as dp
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    Display,
    create_simple_display,
    create_simple_section,
)
from wbcore.metadata.configs.display.instance_display.utils import repeat_field
from wbcore.metadata.configs.display.view_config import DisplayViewConfig

from wbhuman_resources.models.preferences import (
    get_previous_year_balance_expiration_date,
)


class PositionDisplayConfig(DisplayViewConfig):
    def get_instance_display(self) -> Display:
        return create_simple_display(
            [["name", "name", "color"], ["level", "parent", "manager"], [repeat_field(3, "groups")]]
        )

    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="name", label=_("Name")),
                dp.Field(key="level", label=_("Level")),
                dp.Field(key="height", label=_("Height")),
                dp.Field(key="parent", label=_("Parent Position")),
                dp.Field(key="manager", label=_("Manager")),
                dp.Field(key="groups", label=_("Groups")),
            ]
        )


class EmployeeBalanceDisplayConfig(DisplayViewConfig):
    def get_instance_display(self) -> Display:
        return create_simple_display(
            [
                ["profile", "contract_type", "is_active"],
                ["calendar", "position", "enrollment_at"],
                ["extra_days_per_period", "extra_days_frequency", "occupancy_rate"],
                [
                    "available_vacation_balance_previous_year",
                    "available_vacation_balance_current_year",
                    "available_vacation_balance_next_year",
                ],
                [repeat_field(3, "periods_count_per_type_section")],
                [repeat_field(3, "absencerequest_section")],
                [repeat_field(3, "employeeyearbalance_section")],
            ],
            [
                create_simple_section(
                    "periods_count_per_type_section",
                    _("Absence Periods Count per type"),
                    [["periods_count_per_type"]],
                    "periods_count_per_type",
                    collapsed=True,
                ),
                create_simple_section(
                    "absencerequest_section", _("Requests"), [["absencerequest"]], "absencerequest", collapsed=True
                ),
                create_simple_section(
                    "employeeyearbalance_section",
                    _("Balances"),
                    [["employeeyearbalance"]],
                    "employeeyearbalance",
                    collapsed=True,
                ),
            ],
        )

    def get_list_display(self) -> Optional[dp.ListDisplay]:
        current_year = date.today().year

        base_fields = [
            dp.Field(key="profile", label=_("Profile"), width=Unit.PIXEL(250)),
            dp.Field(key="position", label=_("Position"), width=Unit.PIXEL(350)),
            dp.Field(key="contract_type", label=_("Contract Type"), width=Unit.PIXEL(140)),
            dp.Field(key="is_active", label=_("Active"), width=Unit.PIXEL(100)),
            dp.Field(key="occupancy_rate", label=_("Occupancy Rate"), width=Unit.PIXEL(120)),
            dp.Field(key="calendar", label=_("Calendar"), width=Unit.PIXEL(140)),
            dp.Field(key="extra_days_frequency", label=_("Frequency"), width=Unit.PIXEL(120)),
            dp.Field(key="extra_days_per_period", label=_("Periodic days"), width=Unit.PIXEL(120)),
            dp.Field(key="took_long_vacations", label=_("Long Vacation"), width=Unit.PIXEL(140)),
        ]
        if date.today() < get_previous_year_balance_expiration_date(current_year):
            base_fields.append(
                dp.Field(
                    key="available_vacation_balance_previous_year",
                    label=_("Balance ({current_year})").format(current_year=current_year - 1),
                    formatting_rules=[
                        dp.FormattingRule(
                            style={"color": WBColor.RED.value, "fontWeight": "bold"},
                            condition=("<=", 0),
                        ),
                        dp.FormattingRule(
                            style={"color": WBColor.GREEN.value, "fontWeight": "bold"},
                            condition=(">", 0),
                        ),
                    ],
                    width=Unit.PIXEL(175),
                )
            )
        base_fields.append(
            dp.Field(
                key="available_vacation_balance_current_year",
                label=_("Balance ({current_year})").format(current_year=current_year),
                formatting_rules=[
                    dp.FormattingRule(
                        style={"color": WBColor.RED.value, "fontWeight": "bold"},
                        condition=("<=", 0),
                    ),
                    dp.FormattingRule(
                        style={"color": WBColor.GREEN.value, "fontWeight": "bold"},
                        condition=(">", 0),
                    ),
                ],
                width=Unit.PIXEL(175),
            )
        )
        if date.today().month == 12:
            base_fields.append(
                dp.Field(
                    key="available_vacation_balance_next_year",
                    label=_("Balance ({current_year})").format(current_year=current_year + 1),
                    formatting_rules=[
                        dp.FormattingRule(
                            style={"color": WBColor.RED.value, "fontWeight": "bold"},
                            condition=("<=", 0),
                        ),
                        dp.FormattingRule(
                            style={"color": WBColor.GREEN.value, "fontWeight": "bold"},
                            condition=(">", 0),
                        ),
                    ],
                    width=Unit.PIXEL(175),
                )
            )
        return dp.ListDisplay(fields=tuple(base_fields))


class EmployeeDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="profile", label=_("Name")),
                dp.Field(key="calendar", label=_("Calendar")),
                dp.Field(key="contract_type", label=_("Contract")),
                dp.Field(key="position", label=_("Position")),
                dp.Field(key="top_position_repr", label=_("Position N+1")),
                dp.Field(key="position_manager", label=_("Department manager")),
                dp.Field(key="direct_manager", label=_("Direct Manager")),
                dp.Field(key="primary_telephone", label=_("Telephone")),
                dp.Field(key="primary_email", label=_("Email")),
                dp.Field(key="primary_address", label=_("Address")),
                dp.Field(key="enrollment_at", label=_("Since")),
            ]
        )

    def get_instance_display(self) -> Display:
        return create_simple_display(
            [
                ["profile", "is_active", "enrollment_at"],
                ["direct_manager", "calendar", "position"],
                [repeat_field(3, "contract_info_section")],
            ],
            [
                create_simple_section(
                    "contract_info_section",
                    _("Contract Info"),
                    [
                        [
                            "extra_days_frequency",
                            "occupancy_rate",
                            "contract_type",
                        ],
                    ],
                )
            ],
        )


class YearBalanceEmployeeHumanResourceDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="year", label=_("Year")),
                dp.Field(key="_balance", label="Given yearly balance (in hours)"),
                dp.Field(key="_number_mandatory_days_off", label="Mandatory days off (in hours)"),
                dp.Field(key="_total_vacation_hourly_usage", label="Hourly usage (in hours)"),
                dp.Field(key="actual_total_vacation_hourly_balance", label="Hourly available balance (in hours)"),
                dp.Field(key="_balance_in_days", label="Given yearly balance (in days)"),
                dp.Field(key="_number_mandatory_days_off_in_days", label="Mandatory days off (in days)"),
                dp.Field(key="_total_vacation_hourly_usage_in_days", label="Hourly usage (in days)"),
                dp.Field(
                    key="actual_total_vacation_hourly_balance_in_days", label="Hourly available balance (in days)"
                ),
            ]
        )

    def get_instance_display(self) -> Display:
        return create_simple_display(
            [
                ["year", "extra_balance"],
                [repeat_field(2, "hourly_balance_section")],
                [repeat_field(2, "daily_balance_section")],
            ],
            [
                create_simple_section(
                    "hourly_balance_section",
                    _("Balance (in Hours)"),
                    [
                        [
                            "_balance_in_days",
                            "_number_mandatory_days_off_in_days",
                            "_total_vacation_hourly_usage_in_days",
                            "actual_total_vacation_hourly_balance_in_days",
                        ]
                    ],
                    collapsed=False,
                ),
                create_simple_section(
                    "daily_balance_section",
                    _("Balance (in Days)"),
                    [
                        [
                            "_balance",
                            "_number_mandatory_days_off",
                            "_total_vacation_hourly_usage",
                            "actual_total_vacation_hourly_balance",
                        ]
                    ],
                    collapsed=False,
                ),
            ],
        )


class WeeklyOffPeriodEmployeeHumanResourceDisplayConfig(DisplayViewConfig):
    def get_list_display(self) -> Optional[dp.ListDisplay]:
        return dp.ListDisplay(
            fields=[
                dp.Field(key="period", label=_("Period")),
                dp.Field(key="weekday", label=_("Weekday")),
            ]
        )

    def get_instance_display(self) -> Display:
        return create_simple_display([["period", "weekday"]])



from django.db.models import Q
from rest_framework.reverse import reverse
from wbcore.metadata.configs.endpoints import EndpointViewConfig

from wbhuman_resources.models import Review


class ReviewGroupEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return super().get_endpoint()


class ReviewEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        if "pk" in self.view.kwargs:
            obj = Review.objects.get(id=self.view.kwargs.get("pk"))
            if (moderator := obj.moderator) and (moderator != self.request.user.profile):
                return None
            if obj.status in [Review.Status.REVIEW, Review.Status.EVALUATION, Review.Status.VALIDATION]:
                return None
        # Block click on the row to access to the view if it's not an administrator and no review is linked to this person and only has read access
        elif (
            self.request.user in Review.get_administrators()
            and not self.request.user.has_perm("wbcompliance.add_review")
            and self.request.user not in Review.get_administrators()
            and self.view.get_queryset()
            .filter(
                Q(moderator=self.request.user.profile)
                | Q(reviewer=self.request.user.profile)
                | Q(reviewee=self.request.user.profile)
            )
            .count()
            == 0
        ):
            return None
        return super().get_endpoint()

    def get_delete_endpoint(self, **kwargs):
        if "pk" in self.view.kwargs:
            obj = Review.objects.get(id=self.view.kwargs.get("pk"))
            if obj.status in [Review.Status.REVIEW, Review.Status.EVALUATION, Review.Status.VALIDATION]:
                return None
            elif obj.moderator == self.request.user.profile:
                return super().get_delete_endpoint()
        return None


class ReviewReviewGroupEndpointConfig(ReviewEndpointConfig):
    def get_endpoint(self, **kwargs):
        if review_group_id := self.view.kwargs.get("review_group_id", None):
            if "pk" in self.view.kwargs:
                obj = Review.objects.get(id=self.view.kwargs.get("pk"))
                if (moderator := obj.moderator) and (moderator != self.request.user.profile):
                    return None
                if obj.status in [Review.Status.REVIEW, Review.Status.EVALUATION, Review.Status.VALIDATION]:
                    return None
            return reverse("wbhuman_resources:reviewgroup-review-list", args=[review_group_id], request=self.request)
        return None


class ReviewQuestionCategoryEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return super().get_endpoint()


class ReviewQuestionEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return super().get_endpoint()


class ReviewQuestionReviewEndpointConfig(ReviewQuestionEndpointConfig):
    def get_endpoint(self, **kwargs):
        if review_id := self.view.kwargs.get("review_id", None):
            return reverse("wbhuman_resources:review-reviewquestion-list", args=[review_id], request=self.request)
        return None

    def get_create_endpoint(self, **kwargs):
        if review_id := self.view.kwargs.get("review_id", None):
            obj = Review.objects.get(id=review_id)
            if obj.moderator == self.request.user.profile:
                return self.get_endpoint()
        return None

    def get_delete_endpoint(self, **kwargs):
        if review_id := self.view.kwargs.get("review_id", None):
            obj = Review.objects.get(id=review_id)
            if obj.moderator == self.request.user.profile:
                if "pk" in self.view.kwargs:
                    return f'{self.get_endpoint()}{self.view.kwargs["pk"]}/'
                return super().get_delete_endpoint()
        return None


class ReviewQuestionReviewNoCategoryEndpointConfig(ReviewQuestionReviewEndpointConfig):
    def get_endpoint(self, **kwargs):
        if review_id := self.view.kwargs.get("review_id", None):
            return reverse(
                "wbhuman_resources:review-reviewquestionnocategory-list", args=[review_id], request=self.request
            )
        return None


class ReviewQuestionReviewCategoryEndpointConfig(ReviewQuestionReviewEndpointConfig):
    def get_endpoint(self, **kwargs):
        if (review_id := self.view.kwargs.get("review_id", None)) and (
            category_id := self.view.kwargs.get("category_id", None)
        ):
            return reverse(
                "wbhuman_resources:review-reviewquestioncategory-list",
                args=[review_id, category_id],
                request=self.request,
            )
        return None


class ReviewQuestionReviewQuestionCategoryEndpointConfig(ReviewQuestionEndpointConfig):
    def get_endpoint(self, **kwargs):
        if category_id := self.view.kwargs.get("category_id", None):
            return reverse(
                "wbhuman_resources:reviewquestioncategory-reviewquestion-list",
                args=[category_id],
                request=self.request,
            )
        return None


class ReviewAnswerEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return super().get_endpoint()

    def get_instance_endpoint(self, **kwargs):
        if self.instance:
            obj = self.view.get_object()
            if obj.question.review.status in [
                Review.Status.PREPARATION_OF_REVIEW,
                Review.Status.REVIEW,
                Review.Status.EVALUATION,
                Review.Status.VALIDATION,
            ]:
                return None
        return super().get_endpoint()

    def get_create_endpoint(self, **kwargs):
        return None

    def get_delete_endpoint(self, **kwargs):
        return None


class ReviewAnswerReviewNoCategoryEndpointConfig(ReviewAnswerEndpointConfig):
    def get_endpoint(self, **kwargs):
        if review_id := self.view.kwargs.get("review_id", None):
            return reverse(
                "wbhuman_resources:review-reviewanswerquestionnocategory-list", args=[review_id], request=self.request
            )
        return None


class ReviewAnswerReviewQuestionCategoryEndpointConfig(ReviewAnswerEndpointConfig):
    def get_endpoint(self, **kwargs):
        if (review_id := self.view.kwargs.get("review_id", None)) and (
            category_id := self.view.kwargs.get("category_id", None)
        ):
            return reverse(
                "wbhuman_resources:review-reviewanswerquestioncategory-list",
                args=[review_id, category_id],
                request=self.request,
            )
        return None


class ReviewProgressReviewPandasEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return None

    def get_list_endpoint(self, **kwargs):
        return reverse("wbhuman_resources:review-progress-list", [self.view.kwargs["review_id"]], request=self.request)


class ReviewAnswerReviewPandasEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return reverse(
            "wbhuman_resources:review-reviewquestion-list",
            [self.view.kwargs["review_id"]],
            request=self.request,
        )

    def get_list_endpoint(self, **kwargs):
        return reverse(
            "wbhuman_resources:review-reviewanswerpandasview-list",
            [self.view.kwargs["review_id"]],
            request=self.request,
        )


class ReviewProgressPandasEndpointConfig(EndpointViewConfig):
    def get_endpoint(self, **kwargs):
        return None

    def get_list_endpoint(self, **kwargs):
        return reverse("wbhuman_resources:reviewprogress-list", [], request=self.request)

    def get_instance_endpoint(self, **kwargs):
        return reverse("wbhuman_resources:review-list", args=[], request=self.request)



from django.db.models import Q
from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy, pgettext
from rest_framework.reverse import reverse
from wbcore import serializers as wb_serializers
from wbcore.contrib.directory.models import Person
from wbcore.contrib.directory.serializers import PersonRepresentationSerializer
from wbcore.contrib.icons import WBIcon
from wbcore.enums import RequestType
from wbcore.metadata.configs import buttons as bt
from wbcore.metadata.configs.display.instance_display.shortcuts import (
    create_simple_display,
)

from wbhuman_resources.models import Review
from wbhuman_resources.serializers import ReviewModelSerializer


class ReviewButtonConfig(bt.ButtonViewConfig):
    def get_custom_buttons(self):
        if not self.view.kwargs.get("pk", None):
            return {
                bt.WidgetButton(
                    endpoint=reverse("wbhuman_resources:reviewprogress-list", args=[], request=self.request),
                    label=_("Progress"),
                    icon=WBIcon.CHART_BARS_HORIZONTAL.icon,
                )
            }
        return {}

    def get_custom_list_instance_buttons(self):
        return self.get_custom_instance_buttons()

    def get_custom_instance_buttons(self):
        BUTTONS = [
            bt.WidgetButton(key="progress", label=_("Progress"), icon=WBIcon.CHART_BARS_HORIZONTAL.icon),
            bt.ActionButton(
                method=RequestType.PATCH,
                identifiers=("wbhuman_resources:review",),
                key="completelyfilledreviewee",
                action_label=_("Sending Review"),
                title=_("Finish and Send Review"),
                label=_("Finish and Send Review"),
                icon=WBIcon.CONFIRM.icon,
                description_fields=_(
                    "<p>Have you finished filling in the review? </p> <p><span style='color:red'>This action is not reversible</span></p>"
                ),
                confirm_config=bt.ButtonConfig(label=_("Confirm")),
                cancel_config=bt.ButtonConfig(label=pgettext("Non-Transition button", "Cancel")),
            ),
            bt.ActionButton(
                method=RequestType.PATCH,
                identifiers=("wbhuman_resources:review",),
                key="completelyfilledreviewer",
                action_label=_("Sending Review"),
                title=_("Finish and Send Review"),
                label=_("Finish and Send Review"),
                icon=WBIcon.CONFIRM.icon,
                description_fields=_(
                    "<p>Have you finished filling in the review? </p> <p><span style='color:red'>This action is not reversible</span></p>"
                ),
                confirm_config=bt.ButtonConfig(label=_("Confirm")),
                cancel_config=bt.ButtonConfig(label=pgettext("Non-Transition button", "Cancel")),
            ),
            bt.ActionButton(
                method=RequestType.PATCH,
                identifiers=("wbhuman_resources:review",),
                key="signaturereviewee",
                action_label=_("Signing"),
                title=_("Sign"),
                label=_("Sign"),
                icon=WBIcon.CONFIRM.icon,
                description_fields=_(
                    "<p>Do you want to sign the review? </p> <p><span style='color:red'>This action is not reversible</span></p>"
                ),
                confirm_config=bt.ButtonConfig(label=_("Confirm")),
                cancel_config=bt.ButtonConfig(label=pgettext("Non-Transition button", "Cancel")),
                instance_display=create_simple_display([["feedback_reviewee"]]),
            ),
            bt.ActionButton(
                method=RequestType.PATCH,
                identifiers=("wbhuman_resources:review",),
                key="signaturereviewer",
                action_label=_("Signing"),
                title=_("Sign"),
                label=_("Sign"),
                icon=WBIcon.CONFIRM.icon,
                description_fields=_(
                    "<p>Do you want to sign the review? </p> <p><span style='color:red'>This action is not reversible</span></p>"
                ),
                confirm_config=bt.ButtonConfig(label=_("Confirm")),
                cancel_config=bt.ButtonConfig(label=pgettext("Non-Transition button", "Cancel")),
                instance_display=create_simple_display([["feedback_reviewer"]]),
            ),
            bt.ActionButton(
                method=RequestType.PATCH,
                identifiers=("wbhuman_resources:review",),
                key="generate_pdf",
                action_label=_("Sending Report"),
                title=_("Generate and Send Report"),
                label=_("Generate and Send Report"),
                icon=WBIcon.SEND.icon,
                confirm_config=bt.ButtonConfig(label=_("Confirm")),
                cancel_config=bt.ButtonConfig(label=pgettext("Non-Transition button", "Cancel")),
            ),
        ]

        if self.view.kwargs.get("pk", None):
            review = self.view.get_object()

            if review.is_template and review.review_group and review.moderator == self.view.request.user.profile:

                class GenerateReviewModelSerializer(ReviewModelSerializer):
                    employees = wb_serializers.PrimaryKeyRelatedField(
                        queryset=Person.objects.all(),
                        label=gettext_lazy("Employees"),
                        many=True,
                        default=[_employee for _employee in review.review_group.employees.all()],
                    )
                    _employees = PersonRepresentationSerializer(many=True, source="employees")
                    include_kpi = wb_serializers.BooleanField(default=False, label=gettext_lazy("Include KPI"))

                    class Meta:
                        model = Review
                        fields = (
                            "from_date",
                            "to_date",
                            "review_deadline",
                            "auto_apply_deadline",
                            "employees",
                            "_employees",
                            "include_kpi",
                        )

                BUTTONS.append(
                    bt.ActionButton(
                        method=RequestType.PATCH,
                        identifiers=("wbhuman_resources:review",),
                        key="generate",
                        action_label=_("Generating Review"),
                        title=_("Generate Review from Template"),
                        label=_("Generate Review"),
                        icon=WBIcon.DATA_GRID.icon,
                        description_fields=_(
                            "<p>Generate report from :  <p> <b><span>{{computed_str}} </span></b> </p>"
                        ),
                        confirm_config=bt.ButtonConfig(label=_("Confirm")),
                        cancel_config=bt.ButtonConfig(label=pgettext("Non-Transition button", "Cancel")),
                        serializer=GenerateReviewModelSerializer,
                        instance_display=create_simple_display(
                            [
                                ["from_date", "from_date"],
                                ["to_date", "to_date"],
                                ["review_deadline", "review_deadline"],
                                ["auto_apply_deadline", "include_kpi"],
                                ["employees", "employees"],
                            ]
                        ),
                    )
                )
        return {*BUTTONS}


class ReviewGroupButtonConfig(bt.ButtonViewConfig):
    def get_custom_instance_buttons(self):
        class ReviewCounterSerializer(wb_serializers.Serializer):
            counter = wb_serializers.IntegerField(
                read_only=True,
                label=gettext_lazy("Number of reviews found"),
                default=Review.objects.filter(
                    Q(review_group=self.view.get_object())
                    & Q(status=Review.Status.PREPARATION_OF_REVIEW)
                    & Q(moderator=self.view.request.user.profile)
                    & Q(is_template=False)
                ).count(),
            )

        return {
            bt.ActionButton(
                method=RequestType.PATCH,
                identifiers=("wbhuman_resources:reviewgroup",),
                key="submitreviews",
                action_label=_("Submitting Reviews from Stage 1 to Stage 2"),
                title=_("Submit Reviews from Stage 1 to Stage 2"),
                label=_("Submit Reviews"),
                icon=WBIcon.SEND.icon,
                description_fields=_(
                    "<p>Submit Reviews</p> <p>Group : <b>{{name}}</b><p> From : <b>Stage 1: Preparation of review</b></p> <p> To : <b>Stage 2: Fill in review</b></p>  <p style='background-color:skyblue;border-radius:2px;padding:5px;'> <b><span style=color:black> You are going to submit the reviews that are in stage 1 to stage 2 </span></b> </p>"
                ),
                confirm_config=bt.ButtonConfig(label=_("Confirm")),
                cancel_config=bt.ButtonConfig(label=pgettext("Non-Transition button", "Cancel")),
                serializer=ReviewCounterSerializer,
                instance_display=create_simple_display([["counter"]]),
            ),
        }
